<?php

/**
 * Store Locator Plus update manager.
 *
 * Checks remote CSA server for add-on pack updates.
 *
 * @package StoreLocatorPlus\Updates
 * @author Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2012-2016 Charleston Software Associates, LLC
 * 
 * @property        string      $base_path              Base update path.
 * @property        string      $current_version        The current version installed by the user.
 * @property        string      $plugin_slug            Plugin Slug (plugin_directory/plugin_file.php)
 * @property        string      $remote_version         The current production version reported by the CSA server.
 * @property-read   SLPlus      $slplus
 * @property        string      $slug                   Plugin name (plugin_file)
 * @property        string      $update_request_path    The plugin remote update path.
 * @property-read   string      $updater_url            URL of the update server.
 */
class SLPlus_Updates {

    public $base_path;
    public $current_version;
    public $plugin_slug;
    public $remote_version;
    private $slplus;
    public $slug;
    public $update_request_path;
    private $updater_url = 'https://www.storelocatorplus.com/wp-admin/admin-ajax.php';

    /**
     * Initialize a new instance of the WordPress Auto-Update class
     * @param string $current_version
     * @param string $plugin_slug
     */
    function __construct($current_version, $plugin_slug) {
        global $slplus_plugin;

        // Set the class public variables
        $this->slplus = $slplus_plugin;
        $this->current_version = $current_version;
        $this->plugin_slug = $plugin_slug;
        list ($t1, $t2) = explode('/', $plugin_slug);
        $this->slug = str_replace('.php', '', $t2);
        $this->base_path = $this->updater_url;
        $this->set_update_request_path();

        // define the alternative API for updating checking
        add_filter('pre_set_site_transient_update_plugins', array($this, 'check_update'));

        // Define the alternative response for information checking
        add_filter('plugins_api', array($this, 'check_info'), 10, 3);

        // Multisite needs to override WP update notices
        //
        remove_action( 'after_plugin_row_' . $this->plugin_slug, 'wp_plugin_update_row', 10, 2 );
        add_action( 'after_plugin_row_' . $this->plugin_slug, array( $this, 'show_update_notification' ), 10, 2 );        
    }

    /**
     * Add our self-hosted autoupdate plugin to the filter transient
     *
     * @param $transient
     * @return object $ transient
     */
    public function check_update($transient) {
        if ( ! is_object( $transient ) ) {
            $transient = new stdClass;
        }
        
        if ( empty( $transient->checked ) ) {
            return $transient;
        }
        
        // Do not allow single-site updates on multisite, this is for network admins only.
        //
        global $pagenow;
        if ( 'plugins.php' == $pagenow && is_multisite() ) {
            return $transient;
        }

        if ( empty( $transient->response ) || empty( $transient->response[ $this->plugin_slug ] ) ) {
            $this->getRemote_version();

            // If a newer version is available, add the update
            //
            if ( version_compare( $this->current_version , $this->remote_version , '<' ) ) {
                $transient->response[$this->plugin_slug] = $this->get_version_object();
            }
        }
        
        return $transient;
    }
    
    /**
     * Return a version object from the current version info.
     * 
     * Run getRemove_version first.
     * 
     * @return \stdClass
     */
    private function get_version_object() {
        if ( ! isset( $this->remote_version ) || empty( $this->remote_version ) ) {
            $this->getRemote_version();
        }
        
        $obj = new stdClass();
        $obj->slug = $this->slug;
        $obj->new_version = $this->remote_version;
        $obj->url = $this->update_request_path;
        $obj->package = $this->update_request_path;  
        
        return $obj;
    }

    /**
     * Add our self-hosted description to the filter
     *
     * @see https://developer.wordpress.org/reference/functions/plugins_api/
     * 
     * @param mixed $orig original incoming args
     * @param array $action
     * @param object $arg
     * @return bool|object
     */
    public function check_info($orig, $action, $arg) {

        // No slug? Not plugin update.
        //
        if (empty($arg->slug)) {
            return $orig;
        }
        if (!array_key_exists($arg->slug, $this->slplus->add_ons->instances)) {
            return $orig;
        }
        if (!isset($this->slplus->infoFetched[$arg->slug])) {
            $information = $this->getRemote_information($arg->slug);
            $this->slplus->infoFetched[$arg->slug] = true;
            return $information;
        }
        return $orig;
    }

    /**
     * Return the remote version
     * @return string $remote_version
     */
    public function getRemote_version() {
        $request = wp_remote_post(
                $this->update_request_path . '&fetch=version', array(
            'timeout' => '60',
                )
        );
        if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
            $this->remote_version = $request['body'];
        } else {
            $this->remote_version = false;
        }
        return $this->remote_version;
    }

    /**
     * Get information about the remote version
     * @return mixed[] false if cannot get info, unserialized info if we could
     */
    private function getRemote_information($slug = null) {
        $this->set_update_request_path($slug);

        $request = wp_remote_post(
                $this->update_request_path . '&fetch=info', array(
            'timeout' => '60',
                )
        );
        if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
            return unserialize($request['body']);
        }
        return false;
    }

    /**
     * Set the update path.
     */
    private function set_update_request_path($slug = null) {

        // Implied slug, use the update class properties.
        //
        if ($slug === null) {
            $slug = $this->slug;
            $version = $this->current_version;

            // Explicit slug, go look up the info.
        //
        } else {
            $version = $this->slplus->add_ons->instances[$slug]->options['installed_version'];
            if (empty($version)) {
                $version = $this->set_plugin_version($slug);
                ;
            }
        }

        $this->update_request_path = $this->base_path .
                '?action=wpdk_updater' .
                '&surl=' . urlencode(site_url()) .
                '&uid=' . $this->slplus->options_nojs['premium_user_id'] .
                '&sid=' . $this->slplus->options_nojs['premium_subscription_id'] .
                '&target=' . $this->slplus->options_nojs['build_target'] .
                '&slug=' . $slug .
                '&current_version=' . $version;
    }

    /**
     * Set the plugin version if not a registered plugin with options['installed_version'] set.
     *
     * @param $slug
     * @return string
     */
    function set_plugin_version($slug) {
        $version = '00.00.001';

        require_once(ABSPATH . 'wp-admin/includes/plugin-install.php');
        $plugins = get_plugin_updates();
        foreach ((array) $plugins as $plugin_file => $plugin_data) {
            if ($plugin_data->update->slug === $slug) {
                $version = $plugin_data->Version;
                break;
            }
        }

        return $version;
    }
    
    /**
     * show update nofication row -- needed for multisite subsites, because WP won't tell you otherwise!
     *
     * @param string  $file
     * @param array   $plugin
     */
    public function show_update_notification( $file, $plugin ) {
        if( ! current_user_can( 'update_plugins' ) ) {
                return;
        }
        if( ! is_multisite() ) {
                return;
        }
        if ( $this->plugin_slug != $file ) {
                return;
        }
        
        // Remove our filter on the site transient
        remove_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_update' ), 10 );
        
        $update_cache = get_site_transient( 'update_plugins' );
        $update_cache = is_object( $update_cache ) ? $update_cache : new stdClass();
        
        if ( empty( $update_cache->response ) || empty( $update_cache->response[ $this->plugin_slug ] ) ) {
            $cache_key    = md5( 'slp_plugin_' . sanitize_key( $this->plugin_slug ) . '_version_info' );
            $version_info = get_transient( $cache_key );
            if( false === $version_info ) {
                $this->getRemote_version();
                $version_info = $this->get_version_object();
                set_transient( $cache_key, $version_info, 3600 );
            }
            if( ! is_object( $version_info ) ) {
                    return;
            }
            if( version_compare( $this->current_version, $version_info->new_version, '<' ) ) {
                $update_cache->response[ $this->plugin_slug ] = $version_info;
            }
            $update_cache->last_checked = time();
            $update_cache->checked[ $this->plugin_slug ] = $this->current_version;
            set_site_transient( 'update_plugins', $update_cache );
        } else {
            $version_info = $update_cache->response[ $this->plugin_slug ];
        }
        
        // Restore our filter
        add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_update' ) );
        
        if ( ! empty( $update_cache->response[ $this->plugin_slug ] ) && version_compare( $this->current_version, $version_info->new_version, '<' ) ) {
            
                // build a plugin list row, with update notification
                $wp_list_table = _get_list_table( 'WP_Plugins_List_Table' );
                echo '<tr class="plugin-update-tr"><td colspan="' . $wp_list_table->get_column_count() . '" class="plugin-update colspanchange"><div class="update-message">';
                
                $changelog_link = self_admin_url( $this->update_request_path . '&fetch=info' . '&TB_iframe=true&width=772&height=911' );
                
                if ( empty( $version_info->download_link ) ) {
                    printf(
                        __( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s.', 'store-locator-le' ),
                        esc_html( $version_info->slug ),
                        '<a target="_blank" class="thickbox" href="' . esc_url( $changelog_link ) . '">',
                        esc_html( $version_info->new_version ),
                        '</a>'
                    );
                    
                } else {
                    printf(
                        __( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s or %5$supdate now%6$s.', 'store-locator-le' ),
                        esc_html( $version_info->slug ),
                        '<a target="_blank" class="thickbox" href="' . esc_url( $changelog_link ) . '">',
                        esc_html( $version_info->new_version ),
                        '</a>',
                        '<a href="' . esc_url( wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $this->plugin_slug, 'upgrade-plugin_' . $this->plugin_slug ) ) .'">',
                        '</a>'
                    );
                }
                
                do_action( "in_plugin_update_message-{$file}", $plugin, $version_info );
                
                echo '</div></td></tr>';
        }
    }    

}
